@@ -68,28 +68,28 @@ public:
     out_dir_base_ = "gen-json";
   }
 
-  virtual ~t_json_generator() {}
+  ~t_json_generator() override = default;
 
   /**
   * Init and close methods
   */
+  void init_generator() override;
+  void close_generator() override;
+  std::string display_name() const override;
 
-  void init_generator();
-  void close_generator();
-
-  void generate_typedef(t_typedef* ttypedef);
-  void generate_enum(t_enum* tenum);
-  void generate_program();
+  void generate_typedef(t_typedef* ttypedef) override;
+  void generate_enum(t_enum* tenum) override;
+  void generate_program() override;
   void generate_function(t_function* tfunc);
   void generate_field(t_field* field);
 
-  void generate_service(t_service* tservice);
-  void generate_struct(t_struct* tstruct);
+  void generate_service(t_service* tservice) override;
+  void generate_struct(t_struct* tstruct) override;
 
 private:
   bool should_merge_includes_;
 
-  std::ofstream f_json_;
+  ofstream_with_content_based_conditional_update f_json_;
   std::stack<bool> comma_needed_;
 
   template <typename T>
@@ -147,8 +147,8 @@ void t_json_generator::init_generator() {
 
 string t_json_generator::escape_json_string(const string& input) {
   std::ostringstream ss;
-  for (std::string::const_iterator iter = input.begin(); iter != input.end(); iter++) {
-    switch (*iter) {
+  for (char iter : input) {
+    switch (iter) {
     case '\\':
       ss << "\\\\";
       break;
@@ -174,7 +174,7 @@ string t_json_generator::escape_json_string(const string& input) {
       ss << "\\t";
       break;
     default:
-      ss << *iter;
+      ss << iter;
       break;
     }
   }
@@ -245,7 +245,7 @@ void t_json_generator::end_array() {
 
 void t_json_generator::write_type_spec_object(const char* name, t_type* ttype) {
   ttype = ttype->get_true_type();
-  if (ttype->is_struct() || ttype->is_xception() || ttype->is_container()) {
+  if (ttype->is_struct() || ttype->is_xception() || ttype->is_container() || ttype->is_enum()) {
     write_key_and(name);
     start_object(NO_INDENT);
     write_key_and("typeId");
@@ -264,7 +264,18 @@ void t_json_generator::write_type_spec(t_type* ttype) {
 
   write_string(get_type_name(ttype));
 
-  if (ttype->is_struct() || ttype->is_xception()) {
+  if (ttype->annotations_.size() > 0) {
+    write_key_and("annotations");
+    start_object();
+    for (auto & annotation : ttype->annotations_) {
+      for (auto& annotation_value : annotation.second) {
+        write_key_and_string(annotation.first, annotation_value);
+      }
+    }
+    end_object();
+  }
+
+  if (ttype->is_struct() || ttype->is_xception() || ttype->is_enum()) {
     write_key_and_string("class", get_qualified_name(ttype));
   } else if (ttype->is_map()) {
     t_type* ktype = ((t_map*)ttype)->get_key_type();
@@ -273,10 +284,14 @@ void t_json_generator::write_type_spec(t_type* ttype) {
     write_key_and_string("valueTypeId", get_type_name(vtype));
     write_type_spec_object("keyType", ktype);
     write_type_spec_object("valueType", vtype);
-  } else if (ttype->is_list() || ttype->is_set()) {
+  } else if (ttype->is_list()) {
     t_type* etype = ((t_list*)ttype)->get_elem_type();
     write_key_and_string("elemTypeId", get_type_name(etype));
     write_type_spec_object("elemType", etype);
+  } else if (ttype->is_set()) {
+    t_type* etype = ((t_set*)ttype)->get_elem_type();
+    write_key_and_string("elemTypeId", get_type_name(etype));
+    write_type_spec_object("elemType", etype);
   }
 }
 
@@ -357,6 +372,7 @@ void t_json_generator::generate_program() {
     vector<t_program*>::const_iterator inc_it;
     for (inc_it = includes.begin(); inc_it != includes.end(); ++inc_it) {
       write_comma_if_needed();
+      f_json_ << indent();
       write_string((*inc_it)->get_name());
       indicate_comma_needed();
     }
@@ -441,6 +457,16 @@ void t_json_generator::generate_typedef(t_typedef* ttypedef) {
   if (ttypedef->has_doc()) {
     write_key_and_string("doc", ttypedef->get_doc());
   }
+  if (ttypedef->annotations_.size() > 0) {
+    write_key_and("annotations");
+    start_object();
+    for (auto & annotation : ttypedef->annotations_) {
+      for (auto& annotation_value : annotation.second) {
+        write_key_and_string(annotation.first, annotation_value);
+      }
+    }
+    end_object();
+  }
   end_object();
 }
 
@@ -489,8 +515,8 @@ void t_json_generator::write_const_value(t_const_value* value, bool should_force
 
   case t_const_value::CV_MAP: {
     start_object(NO_INDENT);
-    std::map<t_const_value*, t_const_value*> map = value->get_map();
-    std::map<t_const_value*, t_const_value*>::iterator mit;
+    std::map<t_const_value*, t_const_value*, t_const_value::value_compare> map = value->get_map();
+    std::map<t_const_value*, t_const_value*, t_const_value::value_compare>::iterator mit;
     for (mit = map.begin(); mit != map.end(); ++mit) {
       write_comma_if_needed();
       f_json_ << indent();
@@ -540,6 +566,17 @@ void t_json_generator::generate_enum(t_enum* tenum) {
     write_key_and_string("doc", tenum->get_doc());
   }
 
+  if (tenum->annotations_.size() > 0) {
+      write_key_and("annotations");
+      start_object();
+      for (auto & annotation : tenum->annotations_) {
+        for (auto& annotation_value : annotation.second) {
+          write_key_and_string(annotation.first, annotation_value);
+        }
+      }
+      end_object();
+  }
+
   write_key_and("members");
   start_array();
   vector<t_enum_value*> values = tenum->get_constants();
@@ -570,6 +607,17 @@ void t_json_generator::generate_struct(t_struct* tstruct) {
     write_key_and_string("doc", tstruct->get_doc());
   }
 
+  if (tstruct->annotations_.size() > 0) {
+    write_key_and("annotations");
+    start_object();
+    for (auto & annotation : tstruct->annotations_) {
+      for (auto& annotation_value : annotation.second) {
+        write_key_and_string(annotation.first, annotation_value);
+      }
+    }
+    end_object();
+  }
+
   write_key_and_bool("isException", tstruct->is_xception());
 
   write_key_and_bool("isUnion", tstruct->is_union());
@@ -601,6 +649,17 @@ void t_json_generator::generate_service(t_service* tservice) {
     write_key_and_string("doc", tservice->get_doc());
   }
 
+  if (tservice->annotations_.size() > 0) {
+    write_key_and("annotations");
+    start_object();
+    for (auto & annotation : tservice->annotations_) {
+      for (auto& annotation_value : annotation.second) {
+        write_key_and_string(annotation.first, annotation_value);
+      }
+    }
+    end_object();
+  }
+
   write_key_and("functions");
   start_array();
   vector<t_function*> functions = tservice->get_functions();
@@ -629,6 +688,17 @@ void t_json_generator::generate_function(t_function* tfunc) {
     write_key_and_string("doc", tfunc->get_doc());
   }
 
+  if (tfunc->annotations_.size() > 0) {
+    write_key_and("annotations");
+    start_object();
+    for (auto & annotation : tfunc->annotations_) {
+      for (auto& annotation_value : annotation.second) {
+        write_key_and_string(annotation.first, annotation_value);
+      }
+    }
+    end_object();
+  }
+
   write_key_and("arguments");
   start_array();
   vector<t_field*> members = tfunc->get_arglist()->get_members();
@@ -666,6 +736,17 @@ void t_json_generator::generate_field(t_field* field) {
     write_key_and_string("doc", field->get_doc());
   }
 
+  if (field->annotations_.size() > 0) {
+    write_key_and("annotations");
+    start_object();
+    for (auto & annotation : field->annotations_) {
+      for (auto& annotation_value : annotation.second) {
+        write_key_and_string(annotation.first, annotation_value);
+      }
+    }
+    end_object();
+  }
+
   write_key_and("required");
   switch (field->get_req()) {
   case t_field::T_REQUIRED:
@@ -699,7 +780,7 @@ string t_json_generator::get_type_name(t_type* ttype) {
     return "map";
   }
   if (ttype->is_enum()) {
-    return "i32";
+    return "enum";
   }
   if (ttype->is_struct()) {
     return ((t_struct*)ttype)->is_union() ? "union" : "struct";
@@ -722,6 +803,11 @@ string t_json_generator::get_qualified_name(t_type* ttype) {
   return ttype->get_program()->get_name() + "." + ttype->get_name();
 }
 
+std::string t_json_generator::display_name() const {
+  return "JSON";
+}
+
+
 THRIFT_REGISTER_GENERATOR(json,
                           "JSON",
                           "    merge:           Generate output with included files merged\n")
