@@ -75,8 +75,10 @@ public:
   /**
    * Init and close methods
    */
+
   void init_generator() override;
   void close_generator() override;
+  std::string display_name() const override;
 
   void generate_consts(std::vector<t_const*> consts) override;
 
@@ -346,6 +348,8 @@ string t_kotlin_generator::base_type_name(t_base_type* type) {
     return "kotlin.Int";
   case t_base_type::TYPE_I64:
     return "kotlin.Long";
+  case t_base_type::TYPE_UUID:
+    return "java.util.UUID";
   case t_base_type::TYPE_DOUBLE:
     return "kotlin.Double";
   default:
@@ -581,7 +585,7 @@ void t_kotlin_generator::generate_metadata_for_field_annotations(std::ostream& o
     out << "mapOf(" << endl;
     indent_up();
     for (auto& annotation : field->annotations_) {
-      indent(out) << "\"" + annotation.first + "\" to \"" + annotation.second + "\"," << endl;
+      indent(out) << "\"" + annotation.first + "\" to \"" + annotation.second.back() + "\"," << endl;
     }
     indent_down();
     indent(out) << ")";
@@ -1200,6 +1204,8 @@ string t_kotlin_generator::base_type_write_expression(t_base_type* tbase, string
     return "writeI32(" + it + ")";
   case t_base_type::TYPE_I64:
     return "writeI64(" + it + ")";
+  case t_base_type::TYPE_UUID:
+    return "writeUuid(" + it + ")";
   case t_base_type::TYPE_DOUBLE:
     return "writeDouble(" + it + ")";
   default:
@@ -1228,6 +1234,8 @@ string t_kotlin_generator::base_type_read_expression(t_base_type* tbase) {
     return "readI32()";
   case t_base_type::TYPE_I64:
     return "readI64()";
+  case t_base_type::TYPE_UUID:
+    return "readUuid()";
   case t_base_type::TYPE_DOUBLE:
     return "readDouble()";
   default:
@@ -1890,8 +1898,12 @@ string t_kotlin_generator::type_to_enum(t_type* type) {
       return "org.apache.thrift.protocol.TType.I32";
     case t_base_type::TYPE_I64:
       return "org.apache.thrift.protocol.TType.I64";
+    case t_base_type::TYPE_UUID:
+      return "org.apache.thrift.protocol.TType.UUID";
     case t_base_type::TYPE_DOUBLE:
       return "org.apache.thrift.protocol.TType.DOUBLE";
+    default:
+      throw "compiler error: unhandled type";
     }
   } else if (type->is_enum()) {
     return "org.apache.thrift.protocol.TType.I32";
@@ -1995,4 +2007,9 @@ void t_kotlin_generator::generate_kdoc_comment(ostream& out, t_doc* tdoc) {
   }
 }
 
+std::string t_kotlin_generator::display_name() const {
+  return "Kotlin";
+}
+
+
 THRIFT_REGISTER_GENERATOR(kotlin, "Kotlin", "")
