@@ -24,6 +24,7 @@ var THeaderTransport = require('./header_transport');
 module.exports = TFramedTransport;
 
 function TFramedTransport(buffer, callback) {
+  THeaderTransport.call(this);
   this.inBuf = buffer || new Buffer(0);
   this.outBuffers = [];
   this.outCount = 0;
@@ -31,20 +32,20 @@ function TFramedTransport(buffer, callback) {
   this.onFlush = callback;
 };
 
-TFramedTransport.prototype = new THeaderTransport();
+Object.setPrototypeOf(TFramedTransport.prototype, THeaderTransport.prototype);
 
 TFramedTransport.receiver = function(callback, seqid) {
   var residual = [];
-  
+
   return function(data) {
     // push received data to residual
     for(var i = 0; i < data.length; ++i) {
       residual.push(data[i])
     }
 
-    while (residual.length > 0) {      
+    while (residual.length > 0) {
       if (residual.length < 4) {
-		// Not enough bytes to continue, save and resume on next packet
+        // Not enough bytes to continue, save and resume on next packet
         return;
       }
       // get single package sieze
@@ -57,7 +58,7 @@ TFramedTransport.receiver = function(callback, seqid) {
       // splice first 4 bytes
       residual.splice(0, 4)
       // get package data
-      var frame = Buffer.from(residual.splice(0, frameSize));      
+      var frame = Buffer.from(residual.splice(0, frameSize));
       callback(new TFramedTransport(frame), seqid);
     }
   };
