@@ -22,7 +22,8 @@
 #include <stdint.h>
 #include <constants.h>
 #include <struct.h>
-#include "macros.h"
+#include <macros.h>
+#include <bytes.h>
 
 #define LAST_ID(obj) FIX2INT(rb_ary_pop(rb_ivar_get(obj, last_field_id)))
 #define SET_LAST_ID(obj, val) rb_ary_push(rb_ivar_get(obj, last_field_id), val)
@@ -34,10 +35,12 @@ VALUE rb_thrift_compact_proto_native_qmark(VALUE self) {
 static ID last_field_id;
 static ID boolean_field_id;
 static ID bool_value_id;
+static ID rbuf_ivar_id;
 
 static int VERSION;
 static int VERSION_MASK;
 static int TYPE_MASK;
+static int TYPE_BITS;
 static int TYPE_SHIFT_AMOUNT;
 static int PROTOCOL_ID;
 
@@ -165,6 +168,7 @@ static void write_collection_begin(VALUE transport, VALUE elem_type, VALUE size_
 
 VALUE rb_thrift_compact_proto_write_i32(VALUE self, VALUE i32);
 VALUE rb_thrift_compact_proto_write_string(VALUE self, VALUE str);
+VALUE rb_thrift_compact_proto_write_binary(VALUE self, VALUE buf);
 
 VALUE rb_thrift_compact_proto_write_message_end(VALUE self) {
   return Qnil;
@@ -303,9 +307,16 @@ VALUE rb_thrift_compact_proto_write_double(VALUE self, VALUE dub) {
 }
 
 VALUE rb_thrift_compact_proto_write_string(VALUE self, VALUE str) {
+  str = convert_to_utf8_byte_buffer(str);
+  rb_thrift_compact_proto_write_binary(self, str);
+  return Qnil;
+}
+
+VALUE rb_thrift_compact_proto_write_binary(VALUE self, VALUE buf) {
+  buf = force_binary_encoding(buf);
   VALUE transport = GET_TRANSPORT(self);
-  write_varint32(transport, RSTRING_LEN(str));
-  WRITE(transport, RSTRING_PTR(str), RSTRING_LEN(str));
+  write_varint32(transport, RSTRING_LEN(buf));
+  WRITE(transport, StringValuePtr(buf), RSTRING_LEN(buf));
   return Qnil;
 }
 
@@ -316,6 +327,7 @@ VALUE rb_thrift_compact_proto_write_string(VALUE self, VALUE str) {
 #define is_bool_type(ctype) (((ctype) & 0x0F) == CTYPE_BOOLEAN_TRUE || ((ctype) & 0x0F) == CTYPE_BOOLEAN_FALSE)
 
 VALUE rb_thrift_compact_proto_read_string(VALUE self);
+VALUE rb_thrift_compact_proto_read_binary(VALUE self);
 VALUE rb_thrift_compact_proto_read_byte(VALUE self);
 VALUE rb_thrift_compact_proto_read_i32(VALUE self);
 VALUE rb_thrift_compact_proto_read_i16(VALUE self);
@@ -354,8 +366,8 @@ static int8_t get_ttype(int8_t ctype) {
 }
 
 static char read_byte_direct(VALUE self) {
-  VALUE buf = READ(self, 1);
-  return RSTRING_PTR(buf)[0];
+  VALUE byte = rb_funcall(GET_TRANSPORT(self), read_byte_method_id, 0);
+  return (char)(FIX2INT(byte));
 }
 
 static int64_t zig_zag_to_ll(int64_t n) {
@@ -439,7 +451,7 @@ VALUE rb_thrift_compact_proto_read_message_begin(VALUE self) {
     rb_exc_raise(get_protocol_exception(INT2FIX(-1), rb_str_new2(buf)));
   }
   
-  int8_t type = (version_and_type >> TYPE_SHIFT_AMOUNT) & 0x03;
+  int8_t type = (version_and_type >> TYPE_SHIFT_AMOUNT) & TYPE_BITS;
   int32_t seqid = read_varint64(self);
   VALUE messageName = rb_thrift_compact_proto_read_string(self);
   return rb_ary_new3(3, messageName, INT2FIX(type), INT2NUM(seqid));
@@ -458,6 +470,7 @@ VALUE rb_thrift_compact_proto_read_field_begin(VALUE self) {
     
     if (modifier == 0) {
       // not a delta. look ahead for the zigzag varint field id.
+      (void) LAST_ID(self);
       field_id = read_i16(self);
     } else {
       // has a delta. add the delta to the last read field id.
@@ -527,37 +540,46 @@ VALUE rb_thrift_compact_proto_read_double(VALUE self) {
     double f;
     int64_t l;
   } transfer;
-  VALUE bytes = READ(self, 8);
-  uint32_t lo = ((uint8_t)(RSTRING_PTR(bytes)[0]))
-    | (((uint8_t)(RSTRING_PTR(bytes)[1])) << 8)
-    | (((uint8_t)(RSTRING_PTR(bytes)[2])) << 16)
-    | (((uint8_t)(RSTRING_PTR(bytes)[3])) << 24);
-  uint64_t hi = (((uint8_t)(RSTRING_PTR(bytes)[4])))
-    | (((uint8_t)(RSTRING_PTR(bytes)[5])) << 8)
-    | (((uint8_t)(RSTRING_PTR(bytes)[6])) << 16)
-    | (((uint8_t)(RSTRING_PTR(bytes)[7])) << 24);
+  VALUE rbuf = rb_ivar_get(self, rbuf_ivar_id);
+  rb_funcall(GET_TRANSPORT(self), read_into_buffer_method_id, 2, rbuf, INT2FIX(8));
+  uint32_t lo = ((uint8_t)(RSTRING_PTR(rbuf)[0]))
+    | (((uint8_t)(RSTRING_PTR(rbuf)[1])) << 8)
+    | (((uint8_t)(RSTRING_PTR(rbuf)[2])) << 16)
+    | (((uint8_t)(RSTRING_PTR(rbuf)[3])) << 24);
+  uint64_t hi = (((uint8_t)(RSTRING_PTR(rbuf)[4])))
+    | (((uint8_t)(RSTRING_PTR(rbuf)[5])) << 8)
+    | (((uint8_t)(RSTRING_PTR(rbuf)[6])) << 16)
+    | (((uint8_t)(RSTRING_PTR(rbuf)[7])) << 24);
   transfer.l = (hi << 32) | lo;
 
   return rb_float_new(transfer.f);
 }
 
 VALUE rb_thrift_compact_proto_read_string(VALUE self) {
+  VALUE buffer = rb_thrift_compact_proto_read_binary(self);
+  return convert_to_string(buffer);
+}
+
+VALUE rb_thrift_compact_proto_read_binary(VALUE self) {
   int64_t size = read_varint64(self);
   return READ(self, size);
 }
 
 static void Init_constants() {
   thrift_compact_protocol_class = rb_const_get(thrift_module, rb_intern("CompactProtocol"));
-  
+  rb_global_variable(&thrift_compact_protocol_class);
+
   VERSION = rb_num2ll(rb_const_get(thrift_compact_protocol_class, rb_intern("VERSION")));
   VERSION_MASK = rb_num2ll(rb_const_get(thrift_compact_protocol_class, rb_intern("VERSION_MASK")));
   TYPE_MASK = rb_num2ll(rb_const_get(thrift_compact_protocol_class, rb_intern("TYPE_MASK")));
+  TYPE_BITS = rb_num2ll(rb_const_get(thrift_compact_protocol_class, rb_intern("TYPE_BITS")));
   TYPE_SHIFT_AMOUNT = FIX2INT(rb_const_get(thrift_compact_protocol_class, rb_intern("TYPE_SHIFT_AMOUNT")));
   PROTOCOL_ID = FIX2INT(rb_const_get(thrift_compact_protocol_class, rb_intern("PROTOCOL_ID")));
-  
+
   last_field_id = rb_intern("@last_field");
   boolean_field_id = rb_intern("@boolean_field");
   bool_value_id = rb_intern("@bool_value");
+  rbuf_ivar_id = rb_intern("@rbuf");
 }
 
 static void Init_rb_methods() {
@@ -576,6 +598,7 @@ static void Init_rb_methods() {
   rb_define_method(thrift_compact_protocol_class, "write_i64",           rb_thrift_compact_proto_write_i64, 1);
   rb_define_method(thrift_compact_protocol_class, "write_double",        rb_thrift_compact_proto_write_double, 1);
   rb_define_method(thrift_compact_protocol_class, "write_string",        rb_thrift_compact_proto_write_string, 1);
+  rb_define_method(thrift_compact_protocol_class, "write_binary",        rb_thrift_compact_proto_write_binary, 1);
 
   rb_define_method(thrift_compact_protocol_class, "write_message_end", rb_thrift_compact_proto_write_message_end, 0);
   rb_define_method(thrift_compact_protocol_class, "write_struct_begin", rb_thrift_compact_proto_write_struct_begin, 1);
@@ -598,6 +621,7 @@ static void Init_rb_methods() {
   rb_define_method(thrift_compact_protocol_class, "read_i64",            rb_thrift_compact_proto_read_i64, 0);
   rb_define_method(thrift_compact_protocol_class, "read_double",         rb_thrift_compact_proto_read_double, 0);
   rb_define_method(thrift_compact_protocol_class, "read_string",         rb_thrift_compact_proto_read_string, 0);
+  rb_define_method(thrift_compact_protocol_class, "read_binary",         rb_thrift_compact_proto_read_binary, 0);
 
   rb_define_method(thrift_compact_protocol_class, "read_message_end", rb_thrift_compact_proto_read_message_end, 0);
   rb_define_method(thrift_compact_protocol_class, "read_struct_begin",  rb_thrift_compact_proto_read_struct_begin, 0);
@@ -608,58 +632,7 @@ static void Init_rb_methods() {
   rb_define_method(thrift_compact_protocol_class, "read_set_end",       rb_thrift_compact_proto_read_set_end, 0);
 }
 
-static void Init_npmt() {
-  native_proto_method_table *npmt;
-  npmt = ALLOC(native_proto_method_table);
-  
-  npmt->write_field_begin   =  rb_thrift_compact_proto_write_field_begin;
-  npmt->write_field_stop    =  rb_thrift_compact_proto_write_field_stop;
-  npmt->write_map_begin     =  rb_thrift_compact_proto_write_map_begin;
-  npmt->write_list_begin    =  rb_thrift_compact_proto_write_list_begin;
-  npmt->write_set_begin     =  rb_thrift_compact_proto_write_set_begin;
-  npmt->write_byte          =  rb_thrift_compact_proto_write_byte;
-  npmt->write_bool          =  rb_thrift_compact_proto_write_bool;
-  npmt->write_i16           =  rb_thrift_compact_proto_write_i16;
-  npmt->write_i32           =  rb_thrift_compact_proto_write_i32;
-  npmt->write_i64           =  rb_thrift_compact_proto_write_i64;
-  npmt->write_double        =  rb_thrift_compact_proto_write_double;
-  npmt->write_string        =  rb_thrift_compact_proto_write_string;
-  npmt->write_message_end   =  rb_thrift_compact_proto_write_message_end;
-  npmt->write_struct_begin  =  rb_thrift_compact_proto_write_struct_begin;
-  npmt->write_struct_end    =  rb_thrift_compact_proto_write_struct_end;
-  npmt->write_field_end     =  rb_thrift_compact_proto_write_field_end;
-  npmt->write_map_end       =  rb_thrift_compact_proto_write_map_end;
-  npmt->write_list_end      =  rb_thrift_compact_proto_write_list_end;
-  npmt->write_set_end       =  rb_thrift_compact_proto_write_set_end;
-  
-  npmt->read_message_begin  =  rb_thrift_compact_proto_read_message_begin;
-  npmt->read_field_begin    =  rb_thrift_compact_proto_read_field_begin;
-  npmt->read_map_begin      =  rb_thrift_compact_proto_read_map_begin;
-  npmt->read_list_begin     =  rb_thrift_compact_proto_read_list_begin;
-  npmt->read_set_begin      =  rb_thrift_compact_proto_read_set_begin;
-  npmt->read_byte           =  rb_thrift_compact_proto_read_byte;
-  npmt->read_bool           =  rb_thrift_compact_proto_read_bool;
-  npmt->read_i16            =  rb_thrift_compact_proto_read_i16;
-  npmt->read_i32            =  rb_thrift_compact_proto_read_i32;
-  npmt->read_i64            =  rb_thrift_compact_proto_read_i64;
-  npmt->read_double         =  rb_thrift_compact_proto_read_double;
-  npmt->read_string         =  rb_thrift_compact_proto_read_string;
-  npmt->read_message_end    =  rb_thrift_compact_proto_read_message_end;
-  npmt->read_struct_begin   =  rb_thrift_compact_proto_read_struct_begin;
-  npmt->read_struct_end     =  rb_thrift_compact_proto_read_struct_end;
-  npmt->read_field_end      =  rb_thrift_compact_proto_read_field_end;
-  npmt->read_map_end        =  rb_thrift_compact_proto_read_map_end;
-  npmt->read_list_end       =  rb_thrift_compact_proto_read_list_end;
-  npmt->read_set_end        =  rb_thrift_compact_proto_read_set_end;
-  
-  VALUE method_table_object = Data_Wrap_Struct(rb_cObject, 0, free, npmt);
-  rb_const_set(thrift_compact_protocol_class, rb_intern("@native_method_table"), method_table_object);
-}
-
-
-
 void Init_compact_protocol() {
   Init_constants();
   Init_rb_methods();
-  Init_npmt();
 }
